library teamdrt_firebase_package;

import 'dart:async';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:teamdrt_firebase_package/PackageConsts.dart';

/// Login Methods for Firebase
/// Make sure to initialise FirebaseApp before calling these methods
/// These Services will help you help you implement Login and SignUp methods for firebase

class SignInBloc {
  final FirebaseAuth _auth = FirebaseAuth.instance;
  static String verifiationId;
  static int resendToken;

  Stream<User> get user {
    return _auth.authStateChanges();
  }

  var errorController = StreamController<String>.broadcast();
  var signInStatus = StreamController<SignInStatus>.broadcast();

  Future logInWithEmail([
    String email,
    String password,
    ActionCodeSettings actionCodeSettings,
  ]) async {
    try {
      signInStatus.sink.add(SignInStatus.PROCESSING);
      _auth
          .signInWithEmailAndPassword(email: email, password: password)
          .then((value) {
        if (!value.user.emailVerified) {
          value.user.sendEmailVerification(actionCodeSettings);
          signInStatus.sink.add(SignInStatus.EMAILSENT);
        } else {
          if (value.user != null) {
            signInStatus.sink.add(SignInStatus.COMPLETED);
          }
        }
      });
    } on FirebaseException catch (e) {
      failed(e.code);
    } on PlatformException catch (e) {
      failed(e.code);
    } on Exception catch (e) {
      failed(e.toString());
    }
  }

  Future signOut() async {
    _auth.signOut();
  }

  Future loginWithGoogle() async {
    try {
      final GoogleSignIn googleSignIn = GoogleSignIn();
      final GoogleSignInAccount googleSignInAccount =
          await googleSignIn.signIn();
      final GoogleSignInAuthentication googleSignInAuthentication =
          await googleSignInAccount.authentication;
      print(googleSignInAuthentication.accessToken);
      final AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleSignInAuthentication.accessToken,
        idToken: googleSignInAuthentication.idToken,
      );
      _auth.signInWithCredential(credential).then(
            (value) => {
              if (value.user != null)
                {
                  signInStatus.sink.add(SignInStatus.COMPLETED),
                }
            },
          );
    } on PlatformException catch (e) {
      failed(e.code);
    } on FirebaseException catch (e) {
      failed(e.code);
    } catch (e) {
      debugPrint(
        e,
      );
      failed(Constants.unknownError);
    }
  }

  Future sendOtp({String phone, String countryCode}) async {
    assert(phone != null);
    assert(countryCode != null);
    try {
      _auth.verifyPhoneNumber(
          phoneNumber: countryCode + phone,
          verificationCompleted: (credential) {
            verifyOtp(credential1: credential);
            signInStatus.sink.add(SignInStatus.COMPLETED);
          },
          verificationFailed: (e) {
            failed(e.code);
          },
          codeSent: (s, i) {
            verifiationId = s;
            resendToken = i;
            signInStatus.sink.add(SignInStatus.OTPSENT);
          },
          codeAutoRetrievalTimeout: (s) {});
    } on FirebaseAuthException catch (e) {
      failed(e.message);
    }
  }

  Future verifyOtp({String otp, AuthCredential credential1}) async {
    if (credential1 == null) assert(otp != null);
    try {
      var credential = credential1 ??
          PhoneAuthProvider.credential(
              verificationId: verifiationId, smsCode: otp);
      await _auth.signInWithCredential(credential);
      signInStatus.sink.add(SignInStatus.COMPLETED);
    } on FirebaseAuthException catch (e) {
      failed(e.message);
    } catch (e) {
      failed(e.toString());
    }
  }

  failed(String error) {
    print(error);
    errorController.sink.add(error);
    signInStatus.sink.add(SignInStatus.FAILED);
  }

  dispose() {
    errorController.close();
    signInStatus.close();
  }
}

class SignUpBloc {
  final FirebaseAuth _auth = FirebaseAuth.instance;

  Stream<User> get user {
    return _auth.authStateChanges();
  }

  var errorController = StreamController<String>.broadcast();
  var signUpStatus = StreamController<SignInStatus>.broadcast();

  Future signUpWithGoogle() async {
    try {
      signUpStatus.sink.add(SignInStatus.PROCESSING);
      final GoogleSignIn googleSignIn = GoogleSignIn();
      final GoogleSignInAccount googleSignInAccount =
          await googleSignIn.signIn();
      final GoogleSignInAuthentication googleSignInAuthentication =
          await googleSignInAccount.authentication;
      print(googleSignInAuthentication.accessToken);
      final AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleSignInAuthentication.accessToken,
        idToken: googleSignInAuthentication.idToken,
      );
      await _auth.signInWithCredential(credential).then((value) {
        if (value.user != null) {
          signUpStatus.sink.add(SignInStatus.COMPLETED);
        }
      });
    } on PlatformException catch (e) {
      failed(e.code);
    } on FirebaseException catch (e) {
      failed(e.code);
    } catch (e) {
      debugPrint(
        e,
      );
      failed(Constants.unknownError);
    }
  }

  Future signUpWithEmailPassword({
    String email,
    String password,
    String number,
    String userName,
    ActionCodeSettings actionCodeSettings,
  }) async {
    try {
      signUpStatus.sink.add(SignInStatus.PROCESSING);
      await _auth
          .createUserWithEmailAndPassword(email: email, password: password)
          .then(
        (value) {
          if (!value.user.emailVerified) {
            value.user.sendEmailVerification(actionCodeSettings);
            signUpStatus.sink.add(SignInStatus.EMAILSENT);
          }
          value.user.updateProfile(
            displayName: userName,
          );
        },
      );
    } on FirebaseException catch (e) {
      failed(e.code);
    } on PlatformException catch (e) {
      failed(e.code);
    } on Exception catch (e) {
      failed(e.toString());
    }
  }

  failed(String error) {
    print(error);
    errorController.sink.add(error);
    signUpStatus.sink.add(SignInStatus.FAILED);
  }

  dispose() {
    errorController.close();
    signUpStatus.close();
  }
}

enum SignInStatus {
  PROCESSING,
  EMAILSENT,
  COMPLETED,
  OTPSENT,
  FAILED,
}
